mod battery;
mod disk;
mod netspeed;
mod time;
mod temperature;

use comm;
use comm::Channel;
use config::Config;
use std::thread;
use self::battery::BatterySensor;
use self::disk::DiskSensor;
use self::netspeed::NetSpeedSensor;
use self::temperature::TempSensor;
use self::time::TimeSensor;

pub trait Sensor {
    fn icon(&self) -> String;
    fn status(&self) -> String;
    fn process(&mut self);
}

pub fn sensors(tx: &Channel, config: &Config) {
    let zone = config.lookup("sensors.thermal_zone").unwrap();
    let zone = zone.as_str().unwrap();

    let devices = config.lookup("netspeed.devices").unwrap();
    let devices = devices.as_slice().unwrap();
    let devices: Vec<&str> = devices.iter().flat_map(|elem| elem.as_str()).collect();

    let mut sensors: Vec<Box<Sensor>> = vec![
        Box::new(NetSpeedSensor::new(&devices)),
        Box::new(DiskSensor::new("/")),
        Box::new(TempSensor::new(zone)),
        Box::new(TimeSensor::new("%H:%M", true)),
        Box::new(TimeSensor::new("%Y-%m-%d %H:%M", false))
    ];

    let bat = config.lookup("sensors.battery");
    let bat = bat.map(|bat| bat.as_str().unwrap());
    bat.map(|bat| sensors.insert(0, Box::new(BatterySensor::new(bat))));

    loop {
        let status = sensors.iter_mut()
            .map(|sensor| {
                sensor.process();
                format!(
                    "%{{B#666666}} {} %{{B-}} {}",
                    sensor.icon(),
                    sensor.status()
                )
            })
            .collect::<Vec<String>>();

        comm::send(tx, "sensors", &reduce(status));
        thread::sleep_ms(1000);
    }
}

fn reduce(arr: Vec<String>) -> String {
    arr.into_iter().fold("".to_string(), |acc, elem| format!("{} {}", acc, elem))
}
